import es
import cfglib
import playerlib
import popuplib
import sqlite3

# == ADDON INFO =========================================
info = es.AddonInfo()
info.name        = "Make-Your-Own Achievements"
info.version     = "v1.2.0"
info.author      = "The Bash"
info.url         = "http://addons.eventscripts.com/addons/view/myo_achievement"
info.basename    = "myo_achievement"
info.description = "Provides an easy way to add your own achievements by EventScripts."
info.cfgpath     = "config.cfg"
# =======================================================

# == CONFIG FILE CONFIGURATION ==========================
config = cfglib.AddonCFG(es.getAddonPath("myo_achievement") + "/" + info.cfgpath)

config.text("*****************************************") 
config.text("    " + info.name + " " + info.version)
config.text("    Configuration File")
config.text("*****************************************")
config.text("")
config.text("*****************************************")
config.text("    Required variables")
config.text("*****************************************")
 
frontOfMessage        = config.cvar("myoa_front", "[Achievements]", "This is used at the beginning of each message from myo_achievements.")
popupWidth            = config.cvar("myoa_popup_width", 30, "This is the maximum width in characters for each line in the popup.")
dbName                = config.cvar("myoa_db_name", "myo_achievement.sqldb", "This is the name of the database file to use.")
sayCommand            = config.cvar("myoa_popup_command", "!achievements", "This is the command to bring up the popups.")
otherPlayerSayCommand = config.cvar("myoa_others_popup_command", "!otherachievers", "This is the command to bring up the popups to get achievement details of other players.")
top10SayCommand       = config.cvar("myoa_top10_popup_command", "!top10achievers", "This is the command to show the top 10 achievers popup.")

config.write()
# =======================================================

# == CONSOLE COMMAND REGISTRATION =======================
es.regcmd("myoa_add_group", "myo_achievement/addGroupByConsole", "Command for adding achivement groups by console.")
es.regcmd("myoa_add_achievement", "myo_achievement/addAchievementByConsole", "Command for adding achievements by console.")
es.regcmd("myoa_step_progress_player", "myo_achievement/stepProgressPlayerByConsole", "Command for stepping progress by console.")
es.regcmd("myoa_step_progress_list", "myo_achievement/stepProgressListByConsole", "Command for stepping progress by console.")
es.regcmd("myoa_step_progress_all", "myo_achievement/stepProgressAllByConsole", "Command for stepping progress by console.")
es.regcmd("myoa_complete_progress_player", "myo_achievement/completeAchievementPlayerByConsole", "Command for completing progress by console.")
es.regcmd("myoa_complete_progress_list", "myo_achievement/completeAchievementListByConsole", "Command for completing progress by console.")
es.regcmd("myoa_complete_progress_all", "myo_achievement/completeAchievementAllByConsole", "Command for completing progress by console.")
es.regcmd("myoa_reset_progress_player", "myo_achievement/resetAchievementPlayerByConsole", "Command for resetting progress by console.")
es.regcmd("myoa_reset_progress_list", "myo_achievement/resetAchievementListByConsole", "Command for resetting progress by console.")
es.regcmd("myoa_reset_progress_all", "myo_achievement/resetAchievementAllByConsole", "Command for resetting progress by console.")
es.regcmd("myoa_get_player_progress", "myo_achievement/getPlayerProgressByConsole", "Command for returning the player progress by console.")
es.regcmd("myoa_is_player_completed", "myo_achievement/isPlayerCompletedByConsole", "Command for returning if the player has completed by console.")
es.regcmd("myoa_reset_progress", "myo_achievement/resetByConsole", "Command for resetting all stored progress.")
# =======================================================

# == PRIVATE GLOBAL VARIABLES ===========================
_db_connection = None
_db_cursor = None

_achievementGroups = {}
_popupValues = {}
_popupUsers = {}
# =======================================================

# == ES EVENTS ==========================================
def load():
    global _db_connection
    global _db_cursor
       
    es.set("myo_achievement_version",info.version)
    es.makepublic("myo_achievement_version")
    es.loadevents("declare", es.getAddonPath("myo_achievement") + "/achievementevents.res")
    
    config.execute()

    database = es.getAddonPath("myo_achievement") + "/" + dbName
    _db_connection = sqlite3.connect(database)
    _db_cursor = _db_connection.cursor()

    _db_cursor.execute("CREATE TABLE IF NOT EXISTS achievement_groups (id TEXT,name TEXT DEFAULT '',description TEXT DEFAULT '')")
    _db_cursor.execute("CREATE TABLE IF NOT EXISTS achievements (id TEXT,groupid TEXT,name TEXT DEFAULT '', description TEXT DEFAULT '',totalprogress INTEGER DEFAULT '1')")
    _db_cursor.execute("CREATE TABLE IF NOT EXISTS achievements_progress (id INTEGER PRIMARY KEY AUTOINCREMENT,steamid TEXT,groupid TEXT,achievementid TEXT,progress INTEGER DEFAULT '0',name TEXT DEFAULT '')")
    _db_cursor.execute("CREATE TABLE IF NOT EXISTS player_ranks (steamid TEXT PRIMARY KEY,name TEXT DEFAULT '',achievementcount INTEGER DEFAULT '0')")
    
    _db_cursor.execute("DELETE FROM achievement_groups")
    _db_cursor.execute("DELETE FROM achievements")

    if not es.exists("saycommand", sayCommand):
        es.regsaycmd(sayCommand, "myo_achievement/getAchievementGroups", "Shows the active achievements.")
    
    if not es.exists("saycommand", otherPlayerSayCommand):
        es.regsaycmd(otherPlayerSayCommand, "myo_achievement/getOtherPlayerAchievementGroups", "Shows the active achievements for other players.")
        
    if not es.exists("saycommand", top10SayCommand):
        es.regsaycmd(top10SayCommand, "myo_achievement/getTop10Players", "Shows the top 10 achievers.")

def es_map_start(ev):
    es.loadevents(es.getAddonPath("myo_achievement") + "/achievementevents.res")
    _db_connection.commit()

def unload():
    _db_connection.commit()
# =======================================================

# == ACHIEVEMENTGROUP CLASS =============================
class AchievementGroup:
    __id = ""
    __achievements = {}
  
    def __init__(self, newID, newName, newDescription):
        global _achievementGroups
        self.__id = newID
        self.__checkAndSend('WHERE id="%s"' % newID, '(id, name, description) VALUES ("%s", "%s", "%s")' % (newID, newName, newDescription))
        _achievementGroups[newID] = self
        self.__achievements = {}
    
    def __del__(self):
        global _achievementGroups
        del _achievementGroups[self.__id]
        
    def __getattr__(self, name):
        attr = self.__getAttribute(name)
        if attr == None:
            try:
                return self.__dict__[name]
            except KeyError:
                return None
        else:
            return attr
    
    def __setattr__(self, name, value):
        if name == "name" or name == "description":
            self.__setAttribute(name, value)
        else:
            self.__dict__[name] = value
    
    def __getitem__(self, name):
        if name in self.__achievements.keys():
            return self.__achievements[name]
    
    def __setitem__(self, name, value):
        if value[-1] == value[1]:
            self.__achievements[name] = Achievement(self.__id, name, value[0], value[1])
        else:
            self.__achievements[name] = Achievement(self.__id, name, value[0], value[1], value[2])
    
    def __delitem__(self, key):
        if key in self.__achievements.keys():
            del self.__achievements[key]
            
    def __checkAndSend(self, checkQuery, sendQuery):
        _db_cursor.execute("SELECT id FROM achievement_groups %s" % checkQuery)
        if _isCursorEmpty(_db_cursor) == True:
            _db_cursor.execute("INSERT INTO achievement_groups %s" % sendQuery)
    
    def __getAttribute(self, name):
        try:
            _db_cursor.execute('SELECT %s FROM achievement_groups WHERE id="%s"' % (name, self.__id))
            for row in _db_cursor:
                return str(row[0])
                break
            return None
        except sqlite3.OperationalError:
            return None
    
    def __setAttribute(self, name, value):
        _db_cursor.execute('UPDATE achievement_groups SET %s = "%s" WHERE id="%s"' % (name, value, self.__id))
   
    def getAchievements(self):
        return self.__achievements
# =======================================================

# == ACHIEVEMENT CLASS ==================================
class Achievement:
    __parentID = None
    __id = ""
    
    def __init__(self, newParentID, newID, newName, newDescription, newTotalProgress = 1):
        self.__parentID = newParentID
        self.__id = newID
        self.__checkAndSend('WHERE id="%s"' % newID, '(id, groupid, name, description, totalprogress) VALUES ("%s", "%s", "%s", "%s", "%s")' % (newID, newParentID, newName, newDescription, str(newTotalProgress)))

    def __getattr__(self, name):
        attr = self.__getAttribute(name)
        if attr == None:
            try:
                return self.__dict__[name]
            except KeyError:
                return None
        else:
            return attr
    
    def __setattr__(self, name, value):
        if name == "name" or name == "description" or name == "totalprogress":
            self.__setAttribute(name, value)
        else:
            self.__dict__[name] = value

    def stepProgressForPlayer(self, playerID, stepAmount = 1):
        backwardStep = False      
        progress = self.__getPlayerAttribute("progress", playerID)
        totalProgress = int(self.__getAttribute("totalprogress"))
        if not progress == None: 
            progress = int(progress)
            if progress == totalProgress:
                if stepAmount >= 0: 
                    return None
                else:
                    backwardStep = True
            progress = progress + stepAmount
            self.__setPlayerAttribute("progress", playerID, str(progress))
        else:
            progress = stepAmount
            self.__addPlayerAttribute(playerID, progress)
        
        if progress < 0:
            self.__setPlayerAttribute("progress", playerID, "0")
        if progress >= totalProgress:
            self.__setPlayerAttribute("progress", playerID, str(totalProgress))
            self.__doAchievementPlayer(playerID)
        elif backwardStep:
            self.__undoAchievementPlayer(playerID)
    
    def stepProgressForList(self, listFilter, stepAmount = 1):
        for ply in playerlib.getUseridList(listFilter):
            self.stepProgressForPlayer(ply, stepAmount)
    
    def stepProgressForAllActive(self, stepAmount = 1):
        for ply in es.getUseridList():
            self.stepProgressForPlayer(ply, stepAmount)
            
    def completeProgressForPlayer(self, playerID):
        progress = self.__getPlayerAttribute("progress", playerID)
        totalProgress = int(self.__getAttribute("totalprogress"))
        if progress == None:
            self.__addPlayerAttribute(playerID, str(totalProgress))
            self.__doAchievementPlayer(playerID)
        elif int(progress) < self.__totalProgress:
            self.__setPlayerAttribute("progress", playerID, str(totalProgress))
            self.__doAchievementPlayer(playerID)
    
    def completeProgressForList(self, listFilter):
        for ply in playerlib.getUseridList(listFilter):
            self.completeProgressForPlayer(ply)
                
    def completeProgressForAllActive(self):
        for ply in es.getUseridList():
            self.completeProgressForPlayer(ply)
                
    def resetProgressForPlayer(self, playerID):
        progress = self.__getPlayerAttribute("progress", playerID)
        if progress == None:
            self.__addPlayerAttribute(playerID, 0)        
        else:
            if self.__getPlayerAttribute("progress", playerID) == self.__totalprogress:
                self.__setPlayerAttribute("progress", playerID, 0)
                self.__resetAchievementPlayer(playerID)
            else:            
                self.__setPlayerAttribute("progress", playerID, 0)
    
    def resetProgressForList(self, listFilter):
        for ply in playerlib.getUseridList(listFilter):
            self.resetProgressForPlayer(ply, stepAmount)
                
    def resetProgressForAllActive(self):
        for ply in es.getUseridList():
            self.resetProgressForPlayer(ply, stepAmount)
                                          
    def getPlayerProgress(self, playerID):
        return self.__getPlayerAttribute("progress", playerID)
    
    def isPlayerCompleted(self, playerID):
        return self.getPlayerProgress(playerID) == self.__getAttribute("totalprogress")
                
    def __checkAndSend(self, checkQuery, sendQuery):
        _db_cursor.execute("SELECT id FROM achievements %s" % checkQuery)
        if _isCursorEmpty(_db_cursor):
            _db_cursor.execute("INSERT INTO achievements %s" % sendQuery)
            
    def __getAttribute(self, name):
        try:
            _db_cursor.execute('SELECT %s FROM achievements WHERE id="%s" AND groupid="%s"' % (name, self.__id, self.__parentID))
            for row in _db_cursor:
                return str(row[0])
                break
            return None
        except sqlite3.OperationalError:
            return None
                        
    def __setAttribute(self, name, value):
        _db_cursor.execute('UPDATE achievements SET %s = "%s" WHERE id="%s" AND groupid="%s"' % (name, value, self.__id, self.__parentID))
        
    def __addPlayerAttribute(self, playerID, progress):
        playerSteam = _getUniqueSteam(playerID)
        if playerSteam != None:
            _db_cursor.execute('INSERT INTO achievements_progress (steamid, groupid, achievementid, progress, name) VALUES ("%s", "%s", "%s", "%s", "%s")' % (playerSteam, self.__parentID, self.__id, progress, playerlib.getPlayer(playerID).attributes["name"]))   
    
    def __getPlayerAttribute(self, name, playerID):
        playerSteam = _getUniqueSteam(playerID)
        if playerSteam != None:
            _db_cursor.execute('SELECT %s FROM achievements_progress WHERE groupid="%s" AND achievementid="%s" AND steamid="%s"' % (name, self.__parentID, self.__id, playerSteam))
            for row in _db_cursor:
                return str(row[0])
                break
            return None
    
    def __setPlayerAttribute(self, name, playerID, value):
        playerSteam = _getUniqueSteam(playerID)
        if playerSteam != None:
            _db_cursor.execute('UPDATE achievements_progress SET %s = "%s" WHERE groupid="%s" AND achievementid="%s" AND steamid="%s"' % (name, value, self.__parentID, self.__id, playerSteam))
            _db_cursor.execute('UPDATE achievements_progress SET name = "%s" WHERE groupid="%s" AND achievementid="%s" AND steamid="%s"' % (playerlib.getPlayer(playerID).attributes["name"], self.__parentID, self.__id, playerSteam))
            
    def __doAchievementPlayer(self, playerID):
        es.msg("#multi", self.__getMessage("achieved by", playerID))
        self.__triggerEvent("achievement_achieved", playerID)
        try:
            playerSteam = _getUniqueSteam(playerID)
            if playerSteam != None:        
                _db_cursor.execute('SELECT achievementcount FROM player_ranks WHERE steamid = "%s"' % playerSteam)
                for row in _db_cursor:
                    achievementcount = int(row[0]) + 1
                    _db_cursor.execute('UPDATE player_ranks SET achievementcount = "%s" WHERE steamid="%s"' % (achievementcount, playerSteam))
                    _db_cursor.execute('UPDATE player_ranks SET name = "%s" WHERE steamid="%s"' % (playerlib.getPlayer(playerID).attributes["name"], playerSteam))
                    return None
                    break
                _db_cursor.execute('INSERT INTO player_ranks (steamid, achievementcount) VALUES ("%s", "1")' % (playerSteam))
        except sqlite3.OperationalError:
            return None      
    
    def __undoAchievementPlayer(self, playerID):
        es.msg("#multi", self.__getMessage("un-achieved by", playerID))
        self.__triggerEvent("achievement_unachieved", playerID)
    
        try:
            playerSteam = _getUniqueSteam(playerID)
            if playerSteam != None:        
                _db_cursor.execute('SELECT achievementcount FROM player_ranks WHERE steamid = "%s"' % playerSteam)
                for row in _db_cursor:
                    achievementcount = int(row[0]) - 1
                    if achievementcount < 0:
                        achievementcount = 0
                    _db_cursor.execute('UPDATE player_ranks SET achievementcount = "%s" WHERE steamid="%s"' % (achievementcount, playerSteam))
                    _db_cursor.execute('UPDATE player_ranks SET name = "%s" WHERE steamid="%s"' % (playerlib.getPlayer(playerID).attributes["name"], playerSteam))
                    return None
                    break
                _db_cursor.execute('INSERT INTO player_ranks (steamid) VALUES ("%s")' % (playerSteam))      
        except sqlite3.OperationalError:
            return None

    def __resetAchievementPlayer(self, playerID):
        es.msg("#multi", self.__getMessage("reset for", playerID))
        self.__triggerEvent("achievement_reset", playerID)
    
        playerSteam = _getUniqueSteam(playerID)
        if playerSteam != None:        
            _db_cursor.execute('SELECT achievementcount FROM player_ranks WHERE steamid = "%s"' % playerSteam)
            for row in _db_cursor:
                achievementcount = int(row[0]) - 1
                if achievementcount < 0:
                    achievementcount = 0
                _db_cursor.execute('UPDATE player_ranks SET achievementcount = "%s" WHERE steamid="%s"' % (achievementcount, playerSteam))
                _db_cursor.execute('UPDATE player_ranks SET name = "%s" WHERE steamid="%s"' % (playerlib.getPlayer(playerID).attributes["name"], playerSteam))
                return None
                break
            _db_cursor.execute('INSERT INTO player_ranks (steamid, name) VALUES ("%s", "%s")' % (playerSteam, playerlib.getPlayer(playerID).attributes["name"]))      

    def __getMessage(self, action, playerID):
        playerName = playerlib.getPlayer(playerID).attributes["name"]
        return ("#lightgreen%s#default The#green %s Achievement#default has been %s #green%s#default!" % (frontOfMessage, self.__getAttribute("name"), action, playerName))        
        
    def __triggerEvent(self, eventName, playerID):
        es.event("initialize", eventName)
        es.event("setstring", eventName, "achievementgroupid", self.__parentID)
        es.event("setstring", eventName, "achievementid", self.__id)
        es.event("setstring", eventName, "playerid", playerID)
        es.event("fire", eventName)
# =======================================================

# == PRIVATE HELPER FUNCTIONS ===========================
def _isCursorEmpty(cursor):
    for row in cursor:
        return False
        break
    return True

def _getUniqueSteam(playerID):
    if playerID == "0":
        return None
    
    if (str(playerID)).find("STEAM") > -1:
        return playerID
        
    playerSteam = playerlib.getPlayer(playerID).attributes["steamid"]
    if playerSteam.find("LAN") > -1 or playerSteam.find("BOT") > -1:
        playerSteam = playerSteam + playerlib.getPlayer(playerID).attributes["name"]

    return playerSteam
# =======================================================

# == ACHIEVEMENT POPUPS =================================
def getAchievementGroups():
    getPlayerAchievementGroups(es.getcmduserid(), es.getcmduserid())

def getOtherPlayerAchievementGroups():
    if es.getargc() > 1:
        playerList = playerlib.getPlayerList()
        for ply in playerList:
            if ply.attributes["name"] == es.getargv(1):        
                getPlayerAchievementGroups(ply.userid, es.getcmduserid())
                return None
                break
        
        _db_cursor.execute('SELECT * FROM achievements_progress WHERE name="%s"' % es.getargv(1))
        for row in _db_cursor:
            getPlayerAchievementGroups((row[1], row[5]), es.getcmduserid())
            return None
            break
        
        es.tell(es.getcmduserid(), "#multi", "#defaultCould not find a player with the name#green %s" % es.getargv(1))
    else:
        es.tell(es.getcmduserid(), "#multi", "#defaultCommand should be#green %s <playername>" % otherPlayerSayCommand)
                
def getPlayerAchievementGroups(playerID, sendToPlayerID):
    uid = None
    name = None
    if type(playerID) is tuple:
        uid = playerID[0]
        name = playerID[1]
    else:
        uid = playerID
        name = playerlib.getPlayer(uid).attributes["name"]
        
    myPopup = popuplib.easymenu("achievementGroupsMenu", None, getAchievements)
    myPopup.settitle("Achievement Groups - %s" % name)
    _db_cursor.execute('SELECT * FROM achievement_groups')
    groupList = _db_cursor.fetchall()
    for row in groupList:
        count = 0
        completeCount = 0
        achievementDict = _achievementGroups[row[0]].getAchievements()
        for achievement in achievementDict:
            count = count + 1
            if achievementDict[achievement].isPlayerCompleted(uid):
                completeCount = completeCount + 1
        
        if type(playerID) is tuple:
            myPopup.addoption((row[0], (uid, name)), "%s (%d/%d)" % (row[1], completeCount, count))
        else:
            myPopup.addoption((row[0], uid), "%s (%d/%d)" % (row[1], completeCount, count))
    
    myPopup.send(sendToPlayerID)
    myPopup.delete()           

def getAchievements(userid, choice, popupname):
    global _popupUsers
    
    uid = None
    name = None
    
    if type(choice[1]) is tuple:
        uid = choice[1][0]
        name = choice[1][1]
    else:
        uid = choice[1]
        name = playerlib.getPlayer(uid).attributes["name"]
    
    myPopup = popuplib.easymenu("achievementsMenu", None, getAchievement)
    myPopup.settitle("Achievements [%s] - %s" % (_achievementGroups[choice[0]].name, name))
    _db_cursor.execute('SELECT * FROM achievements WHERE groupid="%s"' % choice[0]) 
    achievementList = _db_cursor.fetchall()
    for row in achievementList:
        achievement = _achievementGroups[choice[0]][row[0]]
        try:
            playerProgress = achievement.getPlayerProgress(uid)
            if playerProgress == None:
                playerProgress = "0"
            
            if type(choice[1]) is tuple:
                myPopup.addoption((row[1], row[0], (uid, name)), "%s (%s/%s)" % (row[2], playerProgress, achievement.totalprogress))
            else:
                myPopup.addoption((row[1], row[0], uid), "%s (%s/%s)" % (row[2], playerProgress, achievement.totalprogress))
        except AttributeError:
            pass
    
    _popupUsers[userid] = choice[1]
    myPopup.menuselect = getAchievementEx
    myPopup.send(userid)
    myPopup.delete()

def getAchievementEx(userid, choice, popupname):
    if type(choice) is not tuple:
        if int(choice) == 10:
            getPlayerAchievementGroups(_popupUsers[userid], userid)

def getAchievement(userid, choice, popupname):
    global _popupValues
    global _popupUsers
    
    uid = None
    name = None
    
    if type(choice[2]) is tuple:
        uid = choice[2][0]
        name = choice[2][1]
    else:
        uid = choice[2]
        name = playerlib.getPlayer(uid).attributes["name"]
    
    myPopup = popuplib.create("achievementMenu")    
    achievement = _achievementGroups[choice[0]][choice[1]]
    myPopup.addline(achievement.name + " Achievement")
    myPopup.addline(" ")
    
    descriptionlist = achievement.description.split(" ")
    concat = ""
    length = 0
    for word in descriptionlist:
        length = length + len(word) + 1
        if length > int(popupWidth):
            myPopup.addline(concat)
            concat = ""
            length = len(word) + 1
        concat = concat + word + " "

    myPopup.addline(concat)
    myPopup.addline(" ")        
    
    playerProgress = achievement.getPlayerProgress(uid)
    if playerProgress == None:
        playerProgress = "0"
    
    myPopup.addline("Progress for %s: %s/%s" % (name, playerProgress, achievement.totalprogress))
    myPopup.addline(" ")
    myPopup.addline("0. Go Back")
    
    _popupValues[userid] = choice[0]
    _popupUsers[userid] = choice[2]
    myPopup.menuselect = getAchievementOutEx
    myPopup.send(userid)
    myPopup.delete()

def getAchievementOutEx(userid, choice, popupname):
    if int(choice) == 10:
        getAchievements(userid, (_popupValues[userid], _popupUsers[userid]), popupname)

def getTop10Players():
    myPopup = popuplib.create("top10AchievementMenu")    
    myPopup.addline("Top 10 Achievers")
    myPopup.addline(" ")
    
    _db_cursor.execute('SELECT * FROM player_ranks ORDER BY achievementcount DESC LIMIT 10') 
    achieversList = _db_cursor.fetchall()
    position = 1
    for row in achieversList:
        myPopup.addline("%s. %s (%s achievements)" % (str(position), row[1], row[2]))
        position = position + 1
    
    myPopup.send(es.getcmduserid())
    myPopup.delete()
    
# =======================================================

# == CONSOLE COMMANDS ===================================
def addGroupByConsole():
    if es.getargc() == 4:
        AchievementGroup(es.getargv(1), es.getargv(2), es.getargv(3))
    else:
        _showSyntaxErrorMessage("myoa_add_group", "myoa_add_group groupid groupname groupdescription")

def addAchievementByConsole():
    global _achievementGroups
    if es.getargc() == 5:
        _achievementGroups[es.getargv(1)][es.getargv(2)] = (es.getargv(3), es.getargv(4))
    elif es.getargc() == 6:
        _achievementGroups[es.getargv(1)][es.getargv(2)] = (es.getargv(3), es.getargv(4), int(es.getargv(5)))
    else:
        _showSyntaxErrorMessage("myoa_add_achievement", "myoa_add_achievement groupid achievementid achievementname achievementdescription [totalprogress = 1]")

def stepProgressPlayerByConsole():
    if es.getargc() == 4:
        _achievementGroups[es.getargv(1)][es.getargv(2)].stepProgressForPlayer(es.getargv(3))    
    elif es.getargc() == 5:
        _achievementGroups[es.getargv(1)][es.getargv(2)].stepProgressForPlayer(es.getargv(3), es.getargv(4))
    else:
        _showSyntaxErrorMessage("myoa_step_progress_player", "myoa_step_progress_player groupid achievementid playerid [stepamount = 1]")

def stepProgressListByConsole():
    if es.getargc() == 4:
        _achievementGroups[es.getargv(1)][es.getargv(2)].stepProgressForList(es.getargv(3))    
    elif es.getargc() == 5:
        _achievementGroups[es.getargv(1)][es.getargv(2)].stepProgressForList(es.getargv(3), es.getargv(4))
    else:
        _showSyntaxErrorMessage("myoa_step_progress_list", "myoa_step_progress_list groupid achievementid listfilter [stepamount = 1]")
        
def stepProgressAllByConsole():
    global _achievementGroups
    if es.getargc() == 3:
        _achievementGroups[es.getargv(1)][es.getargv(2)].stepProgressForAllActive()    
    elif es.getargc() == 4:
        _achievementGroups[es.getargv(1)][es.getargv(2)].stepProgressForAllActive(es.getargv(3))
    else:
        _showSyntaxErrorMessage("myoa_step_progress_all", "myoa_step_progress_all groupid achievementid [stepamount = 1]")
        
def completeProgressPlayerByConsole():
    if es.getargc() == 4:
        _achievementGroups[es.getargv(1)][es.getargv(2)].completeProgressForPlayer(es.getargv(3))    
    elif es.getargc() == 5:
        _achievementGroups[es.getargv(1)][es.getargv(2)].completeProgressForPlayer(es.getargv(3), es.getargv(4))
    else:
        _showSyntaxErrorMessage("myoa_complete_progress_player", "myoa_complete_progress_player groupid achievementid playerid")

def completeProgressListByConsole():
    if es.getargc() == 4:
        _achievementGroups[es.getargv(1)][es.getargv(2)].completeProgressForList(es.getargv(3))    
    elif es.getargc() == 5:
        _achievementGroups[es.getargv(1)][es.getargv(2)].completeProgressForList(es.getargv(3), es.getargv(4))
    else:
        _showSyntaxErrorMessage("myoa_complete_progress_list", "myoa_complete_progress_list groupid achievementid listfilter")
        
def completeProgressAllByConsole():
    if es.getargc() == 3:
        _achievementGroups[es.getargv(1)][es.getargv(2)].completeProgressForAllActive()    
    elif es.getargc() == 4:
        _achievementGroups[es.getargv(1)][es.getargv(2)].completeProgressForAllActive(es.getargv(3))
    else:
        _showSyntaxErrorMessage("myoa_complete_progress_all", "myoa_complete_progress_all groupid achievementid")

def resetProgressPlayerByConsole():
    if es.getargc() == 4:
        _achievementGroups[es.getargv(1)][es.getargv(2)].resetProgressForPlayer(es.getargv(3))    
    elif es.getargc() == 5:
        _achievementGroups[es.getargv(1)][es.getargv(2)].resetProgressForPlayer(es.getargv(3), es.getargv(4))
    else:
        _showSyntaxErrorMessage("myoa_reset_progress_player", "myoa_reset_progress_player groupid achievementid playerid")

def resetProgressListByConsole():
    if es.getargc() == 4:
        _achievementGroups[es.getargv(1)][es.getargv(2)].resetProgressForList(es.getargv(3))    
    elif es.getargc() == 5:
        _achievementGroups[es.getargv(1)][es.getargv(2)].resetProgressForList(es.getargv(3), es.getargv(4))
    else:
        _showSyntaxErrorMessage("myoa_reset_progress_list", "myoa_reset_progress_list groupid achievementid listfilter")
        
def resetProgressAllByConsole():
    if es.getargc() == 3:
        _achievementGroups[es.getargv(1)][es.getargv(2)].resetProgressForAllActive()    
    elif es.getargc() == 4:
        _achievementGroups[es.getargv(1)][es.getargv(2)].resetProgressForAllActive(es.getargv(3))
    else:
        _showSyntaxErrorMessage("myoa_reset_progress_all", "myoa_reset_progress_all groupid achievementid")

def getPlayerProgressByConsole():
    if es.getargc() == 4:
        es.set("myoa_player_progress", _achievementGroups[es.getargv(1)][es.getargv(2)].getPlayerProgress(es.getargv(3)))    
    else:
        _showSyntaxErrorMessage("myoa_get_player_progress", "myoa_get_player_progress groupid achievementid playerid")

def isPlayerCompletedByConsole():
    if es.getargc() == 4:
        if _achievementGroups[es.getargv(1)][es.getargv(2)].isPlayerCompleted(es.getargv(3)) == True:
            es.set("myoa_player_complete", 1)
        else:
            es.set("myoa_player_complete", 0)
    else:
        _showSyntaxErrorMessage("myoa_is_player_completed", "myoa_is_player_completed groupid achievementid playerid")

def resetByConsole():
    _db_cursor.execute("DROP TABLE IF EXISTS achievements_progress")
    _db_cursor.execute("DROP TABLE IF EXISTS player_ranks")
    _db_cursor.execute("CREATE TABLE achievements_progress (id INTEGER PRIMARY KEY AUTOINCREMENT,steamid TEXT,groupid TEXT,achievementid TEXT,progress INTEGER DEFAULT '0',name TEXT DEFAULT '')")
    _db_cursor.execute("CREATE TABLE player_ranks (steamid TEXT PRIMARY KEY,name TEXT DEFAULT '',achievementcount INTEGER DEFAULT '0')")    

def _showSyntaxErrorMessage(command, requiredArgs):
    es.dbgmsg(0, "Invalid syntax for %s (wrong number of arguments).\nRequired arguments: %s" % (command, requiredArgs))

# =======================================================